/*
 * Copyright 2021 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.record;

import java.nio.ByteBuffer;
import java.util.Map;

/**
 * Class CompositFastStructWriter dispatches to a number of FastStructWriter's based on actual object type.
 */
public class CompositFastStructWriter implements FastStructWriter<Object> {
	private Map<Class<?>,FastStructWriter<?>> map;
	private ByteBuffer bb;
	private FastStructWriter<?> fsw;
	/**
	 * Create instance of CompositFastStructWriter.
	 * @param map map from Class'es to FastStructWriter's
	 */
	public CompositFastStructWriter(Map<Class<?>,FastStructWriter<?>> map) {
		this(map, 100000);
	}
	/**
	 * Create instance of CompositFastStructWriter.
	 * @param map map from Class'es to FastStructWriter's
	 * @param bufsiz size of byte array to write to
	 */
	public CompositFastStructWriter(Map<Class<?>,FastStructWriter<?>> map, int bufsiz) {
		this.map = map;
		bb = ByteBuffer.allocate(bufsiz);
	}
	@Override
	@SuppressWarnings("unchecked")
	public void write(Object o) throws RecordException {
		fsw = map.get(o.getClass());
		if(fsw == null) {
			throw new RecordException(o.getClass().getName() + " is not mapped to a FastStructWriter");
		}
		fsw.workOn(bb);
		((FastStructWriter<Object>)fsw).write(o);
	}
	@Override
	public void workOn(ByteBuffer bb) {
		this.bb = bb;
	}
	@Override
	public byte[] getBytes() {
		return fsw.getBytes();
	}
	@Override
	public ByteBuffer getByteBuffer() {
		return fsw.getByteBuffer();
	}
}
